"""test code similarities
by defaut docstring are not considered
"""
__revision__ = 'id'
A = 2
B = 3
C = A + B
# need more than X lines to trigger the message
C *= 2
A -= B
# all this should be detected



# pylint: disable=missing-docstring, too-few-public-methods
import datetime


class NewDate(datetime.date):
    @classmethod
    def today(cls):
        return cls(2010, 1, 1)


class Next(object):
    def __init__(self):
        datetime.date = NewDate



"""Verify if constant tests are used inside if statements."""
# pylint: disable=invalid-name, missing-docstring,too-few-public-methods
# pylint: disable=no-init,expression-not-assigned


import collections


def function():
    yield


class Class(object):

    def method(self):
        pass


instance = Class()

if collections: # [using-constant-test]
    pass

# GenExpr
if (node for node in range(10)): # [using-constant-test]
    pass

if lambda: None: # [using-constant-test]
    pass

if function: # [using-constant-test]
    pass

if Class: # [using-constant-test]
    pass

if 2: # [using-constant-test]
    pass

if True: # [using-constant-test]
    pass

if '': # [using-constant-test]
    pass

if b'': # [using-constant-test]
    pass

if 2.0: # [using-constant-test]
    pass

if {}: # [using-constant-test]
    pass

if {1, 2, 3}: # [using-constant-test]
    pass

if (1, 2, 3): # [using-constant-test]
    pass

if (): # [using-constant-test]
    pass

# Generator
generator = function()
if generator: # [using-constant-test]
    pass

if 1 if 2 else 3: # [using-constant-test]
    pass

def test_comprehensions():
    [data for data in range(100) if len] # [using-constant-test]
    [data for data in range(100) if 1] # [using-constant-test]
    (data for data in range(100) if len) # [using-constant-test]
    (data for data in range(100) if 1) # [using-constant-test]
    {data for data in range(100) if len} # [using-constant-test]
    {data: 1 for data in range(100) if len} # [using-constant-test]



# For these, we require to do inference, even though the result can be a
# constant value. For some of them, we could determine that the test
# is constant, such as 2 + 3, but the components of the BinOp
# can be anything else (2 + somefunccall).

name = 42
if name:
    pass

# UnboundMethod / Function
if Class.method:
    pass

# BoundMethod
if instance.method:
    pass

if 3 + 4:
    pass

if 3 and 4:
    pass

if not 3:
    pass

if instance.method():
    pass

# pylint: disable=misplaced-comparison-constant
if 2 < 3:
    pass

if tuple((1, 2, 3)):
    pass

if dict():
    pass

if tuple():
    pass

if [1, 2, 3][:1]:
    pass

def test(*args):
    if args:
        return 42
    return None

def test_good_comprehension_checks():
    [data for data in range(100)]
    [data for data in range(100) if data]
    [data for data in range(100) if len(data)]
    (data for data in range(100) if data)
    (data for data in range(100) if len(data))
    {data for data in range(100) if data}
    {data for data in range(100) if len(data)}
    {data: 1 for data in range(100) if data}
    {data: 1 for data in range(100)}



# -*- coding: utf-8 -*-

# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

"""Looks for overlapping exceptions."""

import astroid

from pylint import interfaces
from pylint import checkers
from pylint.checkers import utils

from pylint.checkers.exceptions import _annotated_unpack_infer


class OverlappingExceptionsChecker(checkers.BaseChecker):
    """Checks for two or more exceptions in the same exception handler
    clause that are identical or parts of the same inheritance hierarchy
    (i.e. overlapping)."""

    __implements__ = interfaces.IAstroidChecker

    name = 'overlap-except'
    msgs = {'W0714': ('Overlapping exceptions (%s)',
                      'overlapping-except',
                      'Used when exceptions in handler overlap or are identical')}
    priority = -2
    options = ()

    @utils.check_messages('overlapping-except')
    def visit_tryexcept(self, node):
        """check for empty except"""
        for handler in node.handlers:
            if handler.type is None:
                continue
            if isinstance(handler.type, astroid.BoolOp):
                continue
            try:
                excs = list(_annotated_unpack_infer(handler.type))
            except astroid.InferenceError:
                continue

            handled_in_clause = []
            for part, exc in excs:
                if exc is astroid.YES:
                    continue
                if (isinstance(exc, astroid.Instance) and
                        utils.inherit_from_std_ex(exc)):
                    # pylint: disable=protected-access
                    exc = exc._proxied

                if not isinstance(exc, astroid.ClassDef):
                    continue

                exc_ancestors = [anc for anc in exc.ancestors()
                                 if isinstance(anc, astroid.ClassDef)]

                for prev_part, prev_exc in handled_in_clause:
                    prev_exc_ancestors = [anc for anc in prev_exc.ancestors()
                                          if isinstance(anc, astroid.ClassDef)]
                    if exc == prev_exc:
                        self.add_message('overlapping-except',
                                         node=handler.type,
                                         args='%s and %s are the same' %
                                         (prev_part.as_string(),
                                          part.as_string()))
                    elif (prev_exc in exc_ancestors or
                          exc in prev_exc_ancestors):
                        ancestor = part if exc in prev_exc_ancestors else prev_part
                        descendant = part if prev_exc in exc_ancestors else prev_part
                        self.add_message('overlapping-except',
                                         node=handler.type,
                                         args='%s is an ancestor class of %s' %
                                         (ancestor.as_string(), descendant.as_string()))
                handled_in_clause += [(part, exc)]


def register(linter):
    """Required method to auto register this checker."""
    linter.register_checker(OverlappingExceptionsChecker(linter))



"""Test that a name is both nonlocal and global in the same scope."""
# pylint: disable=missing-docstring,global-variable-not-assigned,invalid-name,nonlocal-without-binding

def bad(): # [nonlocal-and-global]
    nonlocal missing
    global missing

def good():
    nonlocal missing
    def test():
        global missing
    return test



# pylint: disable=missing-docstring,too-few-public-methods,invalid-name

class A(object):

    def __init__(self):
        self.x = 0
        self.setUp()

    def set_y(self, y):
        self.y = y

    def set_x(self, x):
        self.x = x

    def set_z(self, z):
        self.z = z # [attribute-defined-outside-init]

    def setUp(self):
        self.x = 0
        self.y = 0


class B(A):

    def test(self):
        self.z = 44 # [attribute-defined-outside-init]


class C(object):

    def __init__(self):
        self._init()

    def _init(self):
        self.z = 44


class D(object):

    def setUp(self):
        self.set_z()

    def set_z(self):
        self.z = 42


class E(object):

    def __init__(self):
        i = self._init
        i()

    def _init(self):
        self.z = 44


class Mixin(object):

    def test_mixin(self):
        """Don't emit attribute-defined-outside-init for mixin classes."""
        if self.defined_already: # pylint: disable=access-member-before-definition
            self.defined_already = None



# pylint: disable=no-absolute-import,missing-docstring,import-error,unused-wildcard-import
from indirect1 import * # [wildcard-import]
# This is an unresolved import which still generates the wildcard-import
# warning.
from unknown.package import * # [wildcard-import]



"""Checks import order rule"""
# pylint: disable=unused-import,relative-import,ungrouped-imports,wrong-import-order
# pylint: disable=import-error, too-few-public-methods, missing-docstring,using-constant-test
import os.path

if True:
    from astroid import are_exclusive
try:
    import sys
except ImportError:
    class Myclass(object):
        """docstring"""

if sys.version_info[0] == 3:
    from collections import OrderedDict
else:
    class OrderedDict(object):
        """Nothing to see here."""
        def some_func(self):
            pass

import six  # [wrong-import-position]

CONSTANT = True

import datetime  # [wrong-import-position]

VAR = 0
for i in range(10):
    VAR += i

import scipy  # [wrong-import-position]
import astroid  # [wrong-import-position]



# pylint: disable=E1101, no-absolute-import, import-error,line-too-long, missing-docstring,wrong-import-order,wrong-import-position

try:
    import __builtin__ as builtins
except ImportError:
    import builtins

# Muck up the names in an effort to confuse...
import logging as renamed_logging
import os as logging
from uninferable import UNINFERABLE

FORMAT_STR = '{0}, {1}'

# Statements that should be flagged:
renamed_logging.debug('{0}, {1}'.format(4, 5)) # [logging-format-interpolation]
renamed_logging.log(renamed_logging.DEBUG, 'msg: {}'.format('Run!')) # [logging-format-interpolation]
renamed_logging.debug(FORMAT_STR.format(4, 5)) # [logging-format-interpolation]
renamed_logging.log(renamed_logging.DEBUG, FORMAT_STR.format(4, 5)) # [logging-format-interpolation]
renamed_logging.info("Read {l} rows".format(l=123456789)) # [logging-format-interpolation]

# Statements that should not be flagged:
renamed_logging.debug(format(66, 'x'))
renamed_logging.debug(builtins.format(66, 'x'))
renamed_logging.log(renamed_logging.DEBUG, 'msg: Run!'.upper())
logging.debug('{0}, {1}'.format(4, 5))
logging.log(logging.DEBUG, 'msg: {}'.format('Run!'))
renamed_logging.info("Read {l:,d} rows".format(l=123456789))
renamed_logging.info(UNINFERABLE.format(l=123456789))



"""test mixed tabs and spaces"""
from __future__ import print_function

def spaces_func():
    """yo"""
    print("yo")

def tab_func():
	"""yo""" # [mixed-indentation]
	print("yo") # [mixed-indentation]



"""Test for duplicate-check."""

def main():
    """The second ValueError should be flagged."""
    try:
        raise ValueError('test')
    except ValueError:
        return 1
    except ValueError:  # [duplicate-except]
        return 2
    except (OSError, TypeError):
        return 3



# Copyright (c) 2015-2017 Claudiu Popa <pcmanticore@gmail.com>
# Copyright (c) 2015 Cezar <celnazli@bitdefender.com>
# Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
# Copyright (c) 2017 Martin <MartinBasti@users.noreply.github.com>

# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/PyCQA/pylint/blob/master/COPYING

import contextlib

import astroid

from pylint.checkers import stdlib
from pylint.testutils import CheckerTestCase, Message
from pylint.interfaces import UNDEFINED


@contextlib.contextmanager
def _add_transform(manager, node, transform, predicate=None):
    manager.register_transform(node, transform, predicate)
    try:
        yield
    finally:
        manager.unregister_transform(node, transform, predicate)


class TestStdlibChecker(CheckerTestCase):
    CHECKER_CLASS = stdlib.StdlibChecker

    def test_deprecated_no_qname_on_unexpected_nodes(self):
        # Test that we don't crash on nodes which don't have
        # a qname method. While this test might seem weird since
        # it uses a transform, it's actually testing a crash that
        # happened in production, but there was no way to retrieve
        # the code for which this occurred (how an AssignAttr
        # got to be the result of a function inference
        # beats me..)

        def infer_func(node, context=None):
            new_node = astroid.AssignAttr()
            new_node.parent = node
            yield new_node

        manager = astroid.MANAGER
        transform = astroid.inference_tip(infer_func)
        with _add_transform(manager, astroid.Name, transform):
            node = astroid.extract_node('''
            call_something()
            ''')
            with self.assertNoMessages():
                self.checker.visit_call(node)

    def test_copy_environ(self):
        # shallow copy of os.environ should be reported
        node = astroid.extract_node("""
        import copy, os
        copy.copy(os.environ)
        """)
        with self.assertAddsMessages(
            Message(
                msg_id='shallow-copy-environ', node=node, confidence=UNDEFINED)
        ):
            self.checker.visit_call(node)

    def test_copy_environ_hidden(self):
        # shallow copy of os.environ should be reported
        # hide function names to be sure that checker is not just matching text
        node = astroid.extract_node("""
        from copy import copy as test_cp
        import os as o
        test_cp(o.environ)
        """)
        with self.assertAddsMessages(
            Message(
                msg_id='shallow-copy-environ', node=node, confidence=UNDEFINED)
        ):
            self.checker.visit_call(node)

    def test_copy_dict(self):
        # copy of dict is OK
        node = astroid.extract_node("""
        import copy
        test_dict = {}
        copy.copy(test_dict)
        """)
        with self.assertNoMessages():
            self.checker.visit_call(node)

    def test_copy_uninferable(self):
        # copy of uninferable object should not raise exception, nor make
        # the checker crash
        node = astroid.extract_node("""
        import copy
        from missing_library import MissingObject
        copy.copy(MissingObject)
        """)
        with self.assertNoMessages():
            self.checker.visit_call(node)

    def test_deepcopy_environ(self):
        # deepcopy of os.environ is OK
        node = astroid.extract_node("""
        import copy, os
        copy.deepcopy(os.environ)
        """)
        with self.assertNoMessages():
            self.checker.visit_call(node)



# pylint: disable=old-style-class,missing-docstring,too-few-public-methods
__version__ = "1.0"
SOME_CONSTANT = 42  # [invalid-name]


def say_hello(some_argument):
    return [some_argument * some_value for some_value in range(10)]


class MyClass:  # [invalid-name]
    def __init__(self, arg_x):
        self._my_secret_x = arg_x

    @property
    def my_public_x(self):
        return self._my_secret_x * 2

    def __eq__(self, other):
        return isinstance(other, MyClass) and self.my_public_x == other.my_public_x


def sayHello():  # [invalid-name]
    pass



"""test for Python 2 string formatting error
"""
from __future__ import unicode_literals
# pylint: disable=line-too-long
__revision__ = 1

def pprint_bad():
    """Test string format """
    "{{}}".format(1) # [too-many-format-args]
    "{} {".format() # [bad-format-string]
    "{} }".format() # [bad-format-string]
    "{0} {}".format(1, 2) # [format-combined-specification]
    # +1: [missing-format-argument-key, unused-format-string-argument]
    "{a} {b}".format(a=1, c=2)
    "{} {a}".format(1, 2) # [missing-format-argument-key]
    "{} {}".format(1) # [too-few-format-args]
    "{} {}".format(1, 2, 3) # [too-many-format-args]
    # +1: [missing-format-argument-key,missing-format-argument-key,missing-format-argument-key]
    "{a} {b} {c}".format()
    "{} {}".format(a=1, b=2) # [too-few-format-args]
    # +1: [missing-format-argument-key, missing-format-argument-key]
    "{a} {b}".format(1, 2)



"""
Test PEP 0448 -- Additional Unpacking Generalizations
https://www.python.org/dev/peps/pep-0448/
"""

# pylint: disable=superfluous-parens

UNPACK_TUPLE = (*range(4), 4)
UNPACK_LIST = [*range(4), 4]
UNPACK_SET = {*range(4), 4}
UNPACK_DICT = {'a': 1, **{'b': '2'}}
UNPACK_DICT2 = {**UNPACK_DICT, "x": 1, "y": 2}
UNPACK_DICT3 = {**{'a': 1}, 'a': 2, **{'a': 3}}

UNPACK_IN_COMP = {elem for elem in (*range(10))} # [star-needs-assignment-target]
